@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -27,6 +27,7 @@
 #include <string.h>
 
 #include "misc.h"
+#include "brltty.h"
 
 #include <braille.h>
 
@@ -37,7 +38,6 @@ typedef enum {
 } DriverParameter;
 #define BRLPARMS "device", "driver", "table"
 
-#define BRL_HAVE_VISUAL_DISPLAY
 #include "brl_driver.h"
 #include "tbl.h"
 
@@ -74,6 +74,7 @@ brl_construct(BrailleDisplay *brl, char **parameters, const char *device)
   if(braille_init())
     {
       LogPrint(LOG_INFO, "Libbraille Version: %s", braille_info(BRL_VERSION));
+
       LogPrint(LOG_DEBUG, "Libbraille Installation Directory: %s", braille_info(BRL_PATH));
       LogPrint(LOG_DEBUG, "Libbraille Configuration Directory: %s", braille_info(BRL_PATHCONF));
       LogPrint(LOG_DEBUG, "Libbraille Tables Directory: %s", braille_info(BRL_PATHTBL));
@@ -96,7 +97,7 @@ brl_construct(BrailleDisplay *brl, char **parameters, const char *device)
     }
   else
     {
-      LogPrint(LOG_ERR, "Libbraille initialization erorr: %s", braille_geterror());
+      LogPrint(LOG_DEBUG, "Libbraille initialization erorr: %s", braille_geterror());
     }
   
   return 0;
@@ -108,27 +109,30 @@ brl_destruct(BrailleDisplay *brl)
   braille_close();
 }
 
-static void
-brl_writeVisual(BrailleDisplay *brl)
+static int
+brl_writeWindow(BrailleDisplay *brl, const wchar_t *text)
 {
-  braille_write((char *)brl->buffer, brl->x);
-}
-
-static void
-brl_writeWindow(BrailleDisplay *brl)
-{
-  int i;
-
-  for(i = 0; i < brl->x; i++)
+  if(text)
     {
-      braille_filter(outputTable[brl->buffer[i]], i);
-    }
-  braille_render();
-}
+      char bytes[brl->x];
+      int i;
 
-static void
-brl_writeStatus(BrailleDisplay *brl, const unsigned char *status)
-{
+      for(i = 0; i < brl->x; ++i)
+        {
+          wchar_t character = text[i];
+          bytes[i] = iswLatin1(character)? character: '?';
+        }
+      braille_write(bytes, brl->x);
+
+      if(brl->cursor >= 0)
+        {
+          braille_filter(outputTable[cursorDots()], brl->cursor);
+        }
+
+      braille_render();
+    }
+
+  return 1;
 }
 
 static int
@@ -142,6 +146,7 @@ brl_readCommand(BrailleDisplay *brl, BRL_DriverCommandContext context)
   if(status == -1)
     {
       LogPrint(LOG_ERR, "error in braille_read: %s", braille_geterror());
+      res = BRL_CMD_RESTARTBRL;
     }
   else if(status)
     {
