@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -62,6 +62,7 @@
 #include "misc.h"
 
 #define BRLSTAT ST_MDVStyle
+#define BRL_HAVE_STATUS_CELLS
 #include "brl_driver.h"
 #include "braille.h"
 #include "io_serial.h"
@@ -422,27 +423,28 @@ brl_destruct (BrailleDisplay *brl)
 }
 
 
-static void
+static int
 brl_writeStatus (BrailleDisplay *brl, const unsigned char *s)
 {
   if(nrstatcells >= EXPECTEDNRSTATCELLS)
     memcpy(statbuf, s, EXPECTEDNRSTATCELLS);
   /* BRLTTY expects this display to have only 2 status cells */
+  return 1;
 }
 
 
-static void 
-brl_writeWindow (BrailleDisplay *brl)
+static int 
+brl_writeWindow (BrailleDisplay *brl, const wchar_t *text)
 {
   int i;
   unsigned char *p;
 
   if (brl->x != brl_cols || brl->y != BRLROWS)
-    return;
+    return 1;
     
   if(memcmp(prevdata, brl->buffer, brl_cols) == 0
      && memcmp(prevstatbuf, statbuf, nrstatcells) == 0)
-    return;
+    return 1;
   memcpy(prevdata, brl->buffer, brl_cols);
   memcpy(prevstatbuf, statbuf, nrstatcells);
 
@@ -460,7 +462,7 @@ brl_writeWindow (BrailleDisplay *brl)
 
   if(expect_receive_packet(recvpacket)){
     if(memcmp(recvpacket, ackpacket, ACKPACKETLEN) == 0)
-      return;
+      return 1;
     else{
       packet_to_process = 1;
       LogPrint(LOG_DEBUG, "After sending update, received code %d packet",
@@ -471,6 +473,7 @@ brl_writeWindow (BrailleDisplay *brl)
   memset(prevdata, 0xFF, brl_cols);
   memset(prevstatbuf, 0, nrstatcells);
   /* and then wait for writebrl to be called again. */
+  return 1;
 }
 
 
