@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -59,6 +59,7 @@
 #include "misc.h"
 
 #define BRLSTAT ST_TiemanStyle
+#define BRL_HAVE_STATUS_CELLS
 #include "brl_driver.h"
 #include "braille.h"
 #include "tables.h"		/* for keybindings */
@@ -208,15 +209,16 @@ static void brl_destruct (BrailleDisplay *brl) {
 
 
 
-static void brl_writeStatus (BrailleDisplay *brl, const unsigned char *s) {
+static int brl_writeStatus (BrailleDisplay *brl, const unsigned char *s) {
 	short i;
 
 	/* Dot mapping: */
 	for (i = 0; i < 5; status[i] = outputTable[s[i]], i++);
+	return 1;
 }
 
 
-static void brl_writeWindow (BrailleDisplay *brl) {
+static int brl_writeWindow (BrailleDisplay *brl, const wchar_t *text) {
 	short i;			/* loop counter */
 	unsigned char *pre_data = (unsigned char *)"\002\033Z";	/* bytewise accessible copies */
 	unsigned char *post_data = (unsigned char *)"\001\015";
@@ -259,6 +261,7 @@ static void brl_writeWindow (BrailleDisplay *brl) {
     
 		serialWriteData (MB_serialDevice, rawdata, rawlen);
 	}
+	return 1;
 }
 
 
