@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -19,16 +19,25 @@
 
 #include <stdio.h>
 #include <stdarg.h>
-#include <locale.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
  
+#ifdef HAVE_LIBICUUC
+#include <unicode/uchar.h>
+#endif /* HAVE_LIBICUUC */
+
 #include "misc.h"
+#include "charset.h"
 #include "ctb.h"
 #include "ctb_internal.h"
 #include "brl.h"
 
+typedef struct {
+  BYTE length;
+  wchar_t characters[0XFF];
+} CharacterString;
+
 typedef struct {
   BYTE length;
   BYTE bytes[0XFF];
@@ -40,76 +49,76 @@ static ContractionTableHeader *tableHeader;
 static ContractionTableOffset tableSize;
 static ContractionTableOffset tableUsed;
 
-static const char *const characterClassNames[] = {
-  "space",
-  "letter",
-  "digit",
-  "punctuation",
-  "uppercase",
-  "lowercase",
+static ContractionTableCharacter *characterTable = NULL;
+static int characterTableSize = 0;
+static int characterEntryCount = 0;
+
+static const wchar_t *const characterClassNames[] = {
+  WS_C("space"),
+  WS_C("letter"),
+  WS_C("digit"),
+  WS_C("punctuation"),
+  WS_C("uppercase"),
+  WS_C("lowercase"),
   NULL
 };
 struct CharacterClass {
   struct CharacterClass *next;
   ContractionTableCharacterAttributes attribute;
   BYTE length;
-  char name[1];
+  wchar_t name[1];
 };
 static struct CharacterClass *characterClasses;
 static ContractionTableCharacterAttributes characterClassAttribute;
 
-static const char *const opcodeNames[CTO_None] = {
-  "include",
-  "locale",
+static const wchar_t *const opcodeNames[CTO_None] = {
+  WS_C("include"),
 
-  "capsign",
-  "begcaps",
-  "endcaps",
+  WS_C("capsign"),
+  WS_C("begcaps"),
+  WS_C("endcaps"),
 
-  "letsign",
-  "numsign",
+  WS_C("letsign"),
+  WS_C("numsign"),
 
-  "literal",
-  "replace",
-  "always",
-  "repeated",
+  WS_C("literal"),
+  WS_C("always"),
+  WS_C("repeatable"),
 
-  "largesign",
-  "lastlargesign",
-  "word",
-  "joinword",
-  "lowword",
-  "contraction",
+  WS_C("largesign"),
+  WS_C("lastlargesign"),
+  WS_C("word"),
+  WS_C("joinword"),
+  WS_C("lowword"),
+  WS_C("contraction"),
 
-  "sufword",
-  "prfword",
-  "begword",
-  "begmidword",
-  "midword",
-  "midendword",
-  "endword",
+  WS_C("sufword"),
+  WS_C("prfword"),
+  WS_C("begword"),
+  WS_C("begmidword"),
+  WS_C("midword"),
+  WS_C("midendword"),
+  WS_C("endword"),
 
-  "prepunc",
-  "postpunc",
+  WS_C("prepunc"),
+  WS_C("postpunc"),
 
-  "begnum",
-  "midnum",
-  "endnum",
+  WS_C("begnum"),
+  WS_C("midnum"),
+  WS_C("endnum"),
 
-  "class",
-  "after",
-  "before"
+  WS_C("class"),
+  WS_C("after"),
+  WS_C("before")
 };
 static unsigned char opcodeLengths[CTO_None] = {0};
 
-static char *originalLocale;
-static int noLocale;
-
 typedef struct {
   const char *fileName;
   int lineNumber;		/*line number in table */
 } FileData;
 
+static void compileError (FileData *data, char *format, ...) PRINTF(2, 3);
 static void
 compileError (FileData *data, char *format, ...) {
   char message[0X100];
@@ -154,49 +163,109 @@ allocateBytes (FileData *data, ContractionTableOffset *offset, int count, int al
 }
 
 static int
-saveBytes (FileData *data, ContractionTableOffset *offset, const BYTE *bytes, BYTE length) {
-  if (allocateBytes(data, offset, length+1, __alignof__(BYTE))) {
-    BYTE *address = CTA(tableHeader, *offset);
-    memcpy(address+1, bytes, (*address = length));
+saveBytes (FileData *data, ContractionTableOffset *offset, const void *bytes, int count, int alignment) {
+  if (allocateBytes(data, offset, count, alignment)) {
+    BYTE *address = getContractionTableItem(tableHeader, *offset);
+    memcpy(address, bytes, count);
     return 1;
   }
   return 0;
 }
 
 static int
-saveString (FileData *data, ContractionTableOffset *offset, const char *string) {
-  return saveBytes(data, offset, (const unsigned char *)string, strlen(string));
+saveSequence (FileData *data, ContractionTableOffset *offset, const ByteString *sequence) {
+  if (allocateBytes(data, offset, sequence->length+1, __alignof__(BYTE))) {
+    BYTE *address = getContractionTableItem(tableHeader, *offset);
+    memcpy(address+1, sequence->bytes, (*address = sequence->length));
+    return 1;
+  }
+  return 0;
+}
+
+static ContractionTableCharacter *
+getCharacterEntry (wchar_t character) {
+  int first = 0;
+  int last = characterEntryCount - 1;
+
+  while (first <= last) {
+    int current = (first + last) / 2;
+    ContractionTableCharacter *entry = &characterTable[current];
+
+    if (entry->value < character) {
+      first = current + 1;
+    } else if (entry->value > character) {
+      last = current - 1;
+    } else {
+      return entry;
+    }
+  }
+
+  if (characterEntryCount == characterTableSize) {
+    int newSize = characterTableSize;
+    newSize = newSize? newSize<<1: 0X80;
+
+    {
+      ContractionTableCharacter *newTable = realloc(characterTable, (newSize * sizeof(*newTable)));
+      if (!newTable) return NULL;
+
+      characterTable = newTable;
+      characterTableSize = newSize;
+    }
+  }
+
+  memmove(&characterTable[first+1],
+          &characterTable[first],
+          (characterEntryCount - first) * sizeof(*characterTable));
+  ++characterEntryCount;
+
+  {
+    ContractionTableCharacter *entry = &characterTable[first];
+    memset(entry, 0, sizeof(*entry));
+    entry->value = character;
+    return entry;
+  }
 }
 
 static int
-saveByteString (FileData *data, ContractionTableOffset *offset, const ByteString *string) {
-  return saveBytes(data, offset, string->bytes, string->length);
+saveCharacterTable (void) {
+  ContractionTableOffset offset;
+  if (!characterEntryCount) return 1;
+  if (!saveBytes(NULL, &offset, characterTable,
+                 (tableHeader->characterCount = characterEntryCount) * sizeof(characterTable[0]),
+                 __alignof__(characterTable[0])))
+    return 0;
+
+  tableHeader->characters = offset;
+  return 1;
 }
 
 static ContractionTableRule *
 addRule (
   FileData *data,
   ContractionTableOpcode opcode,
-  ByteString *find,
+  CharacterString *find,
   ByteString *replace,
   ContractionTableCharacterAttributes after,
   ContractionTableCharacterAttributes before
 ) {
   ContractionTableOffset ruleOffset;
-  int ruleSize = sizeof(ContractionTableRule) - 1;
-  if (find) ruleSize += find->length;
+  int ruleSize = sizeof(ContractionTableRule) - sizeof(find->characters[0]);
+  if (find) ruleSize += find->length * sizeof(find->characters[0]);
   if (replace) ruleSize += replace->length;
+
   if (allocateBytes(data, &ruleOffset, ruleSize, __alignof__(ContractionTableRule))) {
-    ContractionTableRule *newRule = CTR(tableHeader, ruleOffset);
+    ContractionTableRule *newRule = getContractionTableItem(tableHeader, ruleOffset);
 
     newRule->opcode = opcode;
     newRule->after = after;
     newRule->before = before;
+
     if (find)
-      memcpy(&newRule->findrep[0], &find->bytes[0],
-             (newRule->findlen = find->length));
+      wmemcpy(&newRule->findrep[0], &find->characters[0],
+              (newRule->findlen = find->length));
     else
       newRule->findlen = 0;
+
     if (replace)
       memcpy(&newRule->findrep[newRule->findlen], &replace->bytes[0],
              (newRule->replen = replace->length));
@@ -208,15 +277,16 @@ addRule (
       ContractionTableOffset *offsetAddress;
 
       if (newRule->findlen == 1) {
-        ContractionTableCharacter *character = &tableHeader->characters[newRule->findrep[0]];
+        ContractionTableCharacter *character = getCharacterEntry(newRule->findrep[0]);
+        if (!character) return NULL;
         if (newRule->opcode == CTO_Always) character->always = ruleOffset;
         offsetAddress = &character->rules;
       } else {
-        offsetAddress = &tableHeader->rules[hash(newRule->findrep)];
+        offsetAddress = &tableHeader->rules[CTH(newRule->findrep)];
       }
 
       while (*offsetAddress) {
-        ContractionTableRule *currentRule = CTR(tableHeader, *offsetAddress);
+        ContractionTableRule *currentRule = getContractionTableItem(tableHeader, *offsetAddress);
         if (newRule->findlen > currentRule->findlen) break;
         if (newRule->findlen == currentRule->findlen) {
           if ((currentRule->opcode == CTO_Always) && (newRule->opcode != CTO_Always)) break;
@@ -234,11 +304,11 @@ addRule (
 }
 
 static const struct CharacterClass *
-findCharacterClass (const unsigned char *name, int length) {
+findCharacterClass (const wchar_t *name, int length) {
   const struct CharacterClass *class = characterClasses;
   while (class) {
     if ((length == class->length) &&
-        (memcmp(name, class->name, length) == 0))
+        (wmemcmp(name, class->name, length) == 0))
       return class;
     class = class->next;
   }
@@ -246,12 +316,12 @@ findCharacterClass (const unsigned char *name, int length) {
 }
 
 static struct CharacterClass *
-addCharacterClass (FileData *data, const unsigned char *name, int length) {
+addCharacterClass (FileData *data, const wchar_t *name, int length) {
   struct CharacterClass *class;
   if (characterClassAttribute) {
-    if ((class = malloc(sizeof(*class) + length - 1))) {
+    if ((class = malloc(sizeof(*class) + ((length - 1) * sizeof(class->name[0]))))) {
       memset(class, 0, sizeof(*class));
-      memcpy(class->name, name, (class->length = length));
+      wmemcpy(class->name, name, (class->length = length));
 
       class->attribute = characterClassAttribute;
       characterClassAttribute <<= 1;
@@ -261,7 +331,7 @@ addCharacterClass (FileData *data, const unsigned char *name, int length) {
       return class;
     }
   }
-  compileError(data, "character class table overflow: %.*s", length, name);
+  compileError(data, "character class table overflow: %.*" PRIws, length, name);
   return NULL;
 }
 
@@ -276,11 +346,11 @@ deallocateCharacterClasses (void) {
 
 static int
 allocateCharacterClasses (void) {
-  const char *const *name = characterClassNames;
+  const wchar_t *const *name = characterClassNames;
   characterClasses = NULL;
   characterClassAttribute = 1;
   while (*name) {
-    if (!addCharacterClass(NULL, (const unsigned char *)*name, strlen(*name))) {
+    if (!addCharacterClass(NULL, *name, wcslen(*name))) {
       deallocateCharacterClasses();
       return 0;
     }
@@ -290,23 +360,23 @@ allocateCharacterClasses (void) {
 }
 
 static ContractionTableOpcode
-getOpcode (FileData *data, const unsigned char *token, int length) {
+getOpcode (FileData *data, const wchar_t *token, int length) {
   ContractionTableOpcode opcode;
   for (opcode=0; opcode<CTO_None; ++opcode)
     if (length == opcodeLengths[opcode])
-      if (memcmp(token, opcodeNames[opcode], length) == 0)
+      if (wmemcmp(token, opcodeNames[opcode], length) == 0)
         return opcode;
-  compileError(data, "opcode not defined: %.*s", length, token);
+  compileError(data, "opcode not defined: %.*" PRIws, length, token);
   return CTO_None;
 }
 
 static int
-getToken (FileData *data, const unsigned char **token, int *length, const char *description) { /*find the next string of contiguous nonblank characters */
-  const unsigned char *start = *token + *length;
+getToken (FileData *data, const wchar_t **token, int *length, const char *description) { /*find the next string of contiguous nonblank characters */
+  const wchar_t *start = *token + *length;
   int count = 0;
 
-  while (isspace(*start)) start++;
-  while (start[count] && !isspace(start[count])) count++;
+  while (iswspace(*start)) start++;
+  while (start[count] && !iswspace(start[count])) count++;
   *token = start;
   if (!(*length = count)) {
     if (description)
@@ -317,103 +387,180 @@ getToken (FileData *data, const unsigned char **token, int *length, const char *
 }				/*token obtained */
 
 static int
-hexadecimalDigit (FileData *data, int *value, char digit) {
-  if (digit >= '0' && digit <= '9')
-    *value = digit - '0';
-  else if (digit >= 'a' && digit <= 'f')
-    *value = digit - 'a' + 10;
-  else if (digit >= 'A' && digit <= 'F')
-    *value = digit - 'A' + 10;
+hexadecimalDigit (FileData *data, int *value, wchar_t digit) {
+  if (digit >= WC_C('0') && digit <= WC_C('9'))
+    *value = digit - WC_C('0');
+  else if (digit >= WC_C('a') && digit <= WC_C('f'))
+    *value = digit - WC_C('a') + 10;
+  else if (digit >= WC_C('A') && digit <= WC_C('F'))
+    *value = digit - WC_C('A') + 10;
   else
     return 0;
   return 1;
 }
 
 static int
-octalDigit (FileData *data, int *value, char digit) {
-  if (digit < '0' || digit > '7') return 0;
-  *value = digit - '0';
+octalDigit (FileData *data, int *value, wchar_t digit) {
+  if (digit < WC_C('0') || digit > WC_C('7')) return 0;
+  *value = digit - WC_C('0');
   return 1;
 }
 
 static int
-parseText (FileData *data, ByteString *result, const unsigned char *token, const int length) {	/*interpret find string */
+parseCharacters (FileData *data, CharacterString *result, const wchar_t *token, const int length) {	/*interpret find string */
   int count = 0;		/*loop counters */
   int index;		/*loop counters */
   for (index = 0; index < length; index++) {
-    char character = token[index];
-    if (character == '\\') { /* escape sequence */
+    wchar_t character = token[index];
+    if (character == WC_C('\\')) { /* escape sequence */
       int ok = 0;
       int start = index;
       if (++index < length) {
         switch (character = token[index]) {
-          case '\\':
+          case WC_C('\\'):
             ok = 1;
             break;
-          case 'f':
-            character = '\f';
+
+          case WC_C('f'):
+            character = WC_C('\f');
             ok = 1;
             break;
-          case 'n':
-            character = '\n';
+
+          case WC_C('n'):
+            character = WC_C('\n');
             ok = 1;
             break;
-          case 'o':
-            if (length - index > 3) {
-              int high, middle, low;
-              if (octalDigit(data, &high, token[++index]))
-                if (high < 04)
-                  if (octalDigit(data, &middle, token[++index]))
-                    if (octalDigit(data, &low, token[++index])) {
-                      character = (high << 6) | (middle << 3) | low;
-                      ok = 1;
-                    }
-            }
-            break;
-          case 'r':
-            character = '\r';
+
+          case WC_C('r'):
+            character = WC_C('\r');
             ok = 1;
             break;
-          case 's':
-            character = ' ';
+
+          case WC_C('s'):
+            character = WC_C(' ');
             ok = 1;
             break;
-          case 't':
-            character = '\t';
+
+          case WC_C('t'):
+            character = WC_C('\t');
             ok = 1;
             break;
-          case 'v':
-            character = '\v';
+
+          case WC_C('v'):
+            character = WC_C('\v');
             ok = 1;
             break;
-          case 'x':
-            if (length - index > 2) {
-              int high, low;
-              if (hexadecimalDigit(data, &high, token[++index]))
-                if (hexadecimalDigit(data, &low, token[++index])) {
-                  character = (high << 4) | low;
-                  ok = 1;
+
+          {
+            int count;
+
+          case WC_C('o'):
+            count = 3;
+
+            if ((length - index) > count) {
+              character = 0;
+              ok = 1;
+
+              do {
+                int octet;
+                if (!octalDigit(data, &octet, token[++index])) {
+                  ok = 0;
+                  break;
                 }
+
+                character = (character << 3) | octet;
+              } while (--count);
             }
             break;
+          }
+
+          {
+            int count;
+
+          case WC_C('U'):
+            count = 8;
+            goto hexadecimal;
+
+          case WC_C('u'):
+            count = 4;
+            goto hexadecimal;
+
+          case WC_C('x'):
+            count = 2;
+          hexadecimal:
+
+            if ((length - index) > count) {
+              character = 0;
+              ok = 1;
+
+              do {
+                int nibble;
+                if (!hexadecimalDigit(data, &nibble, token[++index])) {
+                  ok = 0;
+                  break;
+                }
+
+                character = (character << 4) | nibble;
+              } while (--count);
+            }
+            break;
+          }
+
+          case WC_C('<'): {
+            const wchar_t *first = &token[++index];
+            const wchar_t *end = wmemchr(first, WC_C('>'), length-index);
+
+            if (end) {
+              int count = end - first;
+              index += count;
+
+              {
+                char name[count+1];
+
+                {
+                  int i;
+                  for (i=0; i<count; i+=1) {
+                    wchar_t wc = first[i];
+                    if (wc == WC_C('_')) wc = WC_C(' ');
+                    if (!iswLatin1(wc)) goto badName;
+                    name[i] = wc;
+                  }
+                }
+                name[count] = 0;
+
+#ifdef HAVE_LIBICUUC
+                {
+                  UErrorCode error = U_ZERO_ERROR;
+                  character = u_charFromName(U_EXTENDED_CHAR_NAME, name, &error);
+                  if (U_SUCCESS(error)) ok = 1;
+                }
+#endif /* HAVE_LIBICUUC */
+              }
+            } else {
+              index = length - 1;
+            }
+
+          badName:
+            break;
+          }
         }
       }
       if (!ok) {
         index++;
-        compileError(data, "invalid escape sequence: %.*s",
+        compileError(data, "invalid escape sequence: %.*" PRIws,
                      index-start, &token[start]);
         return 0;
       }
     }
-    if (!character) character = ' ';
-    result->bytes[count++] = character;
+    if (!character) character = WC_C(' ');
+    result->characters[count++] = character;
   }
   result->length = count;
   return 1;
 }				/*find string interpreted */
 
 static int
-parseDots (FileData *data, ByteString *cells, const unsigned char *token, const int length) {	/*get dot patterns */
+parseDots (FileData *data, ByteString *cells, const wchar_t *token, const int length) {	/*get dot patterns */
   BYTE cell = 0;		/*assembly place for dots */
   int count = 0;		/*loop counters */
   int index;		/*loop counters */
@@ -421,48 +568,48 @@ parseDots (FileData *data, ByteString *cells, const unsigned char *token, const
 
   for (index = 0; index < length; index++) {
     int started = index != start;
-    char character = token[index];
+    wchar_t character = token[index];
     switch (character) { /*or dots to make up Braille cell */
       {
         int dot;
-      case '1':
+      case WC_C('1'):
         dot = BRL_DOT1;
         goto haveDot;
-      case '2':
+      case WC_C('2'):
         dot = BRL_DOT2;
         goto haveDot;
-      case '3':
+      case WC_C('3'):
         dot = BRL_DOT3;
         goto haveDot;
-      case '4':
+      case WC_C('4'):
         dot = BRL_DOT4;
         goto haveDot;
-      case '5':
+      case WC_C('5'):
         dot = BRL_DOT5;
         goto haveDot;
-      case '6':
+      case WC_C('6'):
         dot = BRL_DOT6;
         goto haveDot;
-      case '7':
+      case WC_C('7'):
         dot = BRL_DOT7;
         goto haveDot;
-      case '8':
+      case WC_C('8'):
         dot = BRL_DOT8;
       haveDot:
         if (started && !cell) goto invalid;
         if (cell & dot) {
-          compileError(data, "dot specified more than once: %c", character);
+          compileError(data, "dot specified more than once: %.1" PRIws, &character);
           return 0;
         }
         cell |= dot;
         break;
       }
-      case '0':			/*blank */
+      case WC_C('0'):			/*blank */
         if (started) goto invalid;
         break;
-      case '-':			/*got all dots for this cell */
+      case WC_C('-'):			/*got all dots for this cell */
         if (!started) {
-          compileError(data, "missing cell specification: %.*s",
+          compileError(data, "missing cell specification: %.*" PRIws,
                        length-index, &token[index]);
           return 0;
         }
@@ -472,7 +619,7 @@ parseDots (FileData *data, ByteString *cells, const unsigned char *token, const
         break;
       default:
       invalid:
-        compileError(data, "invalid dot number: %c", character);
+        compileError(data, "invalid dot number: %.1" PRIws, &character);
         return 0;
     }
   }
@@ -486,33 +633,25 @@ parseDots (FileData *data, ByteString *cells, const unsigned char *token, const
 }				/*end of function parseDots */
 
 static int
-getCharacters (FileData *data, ByteString *characters, const unsigned char **token, int *length) {
+getCharacters (FileData *data, CharacterString *characters, const wchar_t **token, int *length) {
   if (getToken(data, token, length, "characters"))
-    if (parseText(data, characters, *token, *length))
+    if (parseCharacters(data, characters, *token, *length))
       return 1;
   return 0;
 }
 
 static int
-getFindText (FileData *data, ByteString *find, const unsigned char **token, int *length) {
+getFindText (FileData *data, CharacterString *find, const wchar_t **token, int *length) {
   if (getToken(data, token, length, "find text"))
-    if (parseText(data, find, *token, *length))
+    if (parseCharacters(data, find, *token, *length))
       return 1;
   return 0;
 }
 
 static int
-getReplaceText (FileData *data, ByteString *replace, const unsigned char **token, int *length) {
-  if (getToken(data, token, length, "replacement text"))
-    if (parseText(data, replace, *token, *length))
-      return 1;
-  return 0;
-}
-
-static int
-getReplacePattern (FileData *data, ByteString *replace, const unsigned char **token, int *length) {
+getReplacePattern (FileData *data, ByteString *replace, const wchar_t **token, int *length) {
   if (getToken(data, token, length, "replacement pattern")) {
-    if (*length == 1 && **token == '=') {
+    if (*length == 1 && **token == WC_C('=')) {
       replace->length = 0;
       return 1;
     }
@@ -522,48 +661,40 @@ getReplacePattern (FileData *data, ByteString *replace, const unsigned char **to
   return 0;
 }
 
-static const char *
-setLocale (const char *locale) {
-  return setlocale(LC_CTYPE, locale);
-}
-
 static int
-getCharacterClass (FileData *data, const struct CharacterClass **class, const unsigned char **token, int *length) {
+getCharacterClass (FileData *data, const struct CharacterClass **class, const wchar_t **token, int *length) {
   if (getToken(data, token, length, "character class name")) {
     if ((*class = findCharacterClass(*token, *length))) return 1;
-    compileError(data, "character class not defined: %.*s", *length, *token);
+    compileError(data, "character class not defined: %.*" PRIws, *length, *token);
   }
   return 0;
 }
 
-static const char *
-getLocale (void) {
-  return setLocale(NULL);
-}
-
 static int processFile (const char *fileName);
 static int
-includeFile (FileData *data, ByteString *path) {
+includeFile (FileData *data, CharacterString *path) {
   const char *prefixAddress = data->fileName;
   int prefixLength = 0;
-  const unsigned char *suffixAddress = path->bytes;
+  const wchar_t *suffixAddress = path->characters;
   int suffixLength = path->length;
-  if (*suffixAddress != '/') {
+
+  if (*suffixAddress != WC_C('/')) {
     const char *ptr = strrchr(prefixAddress, '/');
     if (ptr) prefixLength = ptr - prefixAddress + 1;
   }
+
   {
     char file[prefixLength + suffixLength + 1];
-    snprintf(file, sizeof(file), "%.*s%.*s",
+    snprintf(file, sizeof(file), "%.*s%.*" PRIws,
              prefixLength, prefixAddress, suffixLength, suffixAddress);
     return processFile(file);
   }
 }
 
 static int
-processLine (FileData *data, const char *line) {
+processWcharLine (FileData *data, const wchar_t *line) {
   int ok = 0;
-  const unsigned char *token = (const unsigned char *)line;
+  const wchar_t *token = line;
   int length = 0;			/*length of token */
   ContractionTableOpcode opcode;
   ContractionTableCharacterAttributes after = 0;
@@ -571,7 +702,7 @@ processLine (FileData *data, const char *line) {
 
 doOpcode:
   if (!getToken(data, &token, &length, NULL)) return 1;			/*blank line */
-  if (*token == '#') return 1;
+  if (*token == WC_C('#')) return 1;
   opcode = getOpcode(data, token, length);
 
   switch (opcode) { /*Carry out operations */
@@ -579,36 +710,19 @@ doOpcode:
       break;
 
     case CTO_IncludeFile: {
-      ByteString path;
+      CharacterString path;
       if (getToken(data, &token, &length, "include file path"))
-        if (parseText(data, &path, token, length))
+        if (parseCharacters(data, &path, token, length))
           if (!includeFile(data, &path))
             goto failure;
       break;
     }
 
-    case CTO_Locale: {
-      ByteString locale;
-      if (getToken(data, &token, &length, "locale specification"))
-        if (parseText(data, &locale, token, length)) {
-          char string[locale.length + 1];
-          snprintf(string, sizeof(string), "%.*s", locale.length, locale.bytes);
-          if (strcmp(string, "-") == 0) {
-            noLocale = 1;
-          } else if (!setLocale(string)) {
-            compileError(data, "locale not available: %s", string);
-          } else {
-            noLocale = 0;
-          }
-        }
-      break;
-    }
-
     case CTO_Always:
     case CTO_LargeSign:
     case CTO_LastLargeSign:
     case CTO_WholeWord:
-    case CTO_JoinableWord:
+    case CTO_JoinedWord:
     case CTO_LowWord:
     case CTO_SuffixableWord:
     case CTO_PrefixableWord:
@@ -622,8 +736,8 @@ doOpcode:
     case CTO_BegNum:
     case CTO_MidNum:
     case CTO_EndNum:
-    case CTO_Repeated: {
-      ByteString find;
+    case CTO_Repeatable: {
+      CharacterString find;
       ByteString replace;
       if (getFindText(data, &find, &token, &length))
         if (getReplacePattern(data, &replace, &token, &length))
@@ -632,19 +746,9 @@ doOpcode:
       break;
     }
 
-    case CTO_Replace: {
-      ByteString find;
-      ByteString replace;
-      if (getFindText(data, &find, &token, &length))
-        if (getReplaceText(data, &replace, &token, &length))
-          if (!addRule(data, opcode, &find, &replace, after, before))
-            goto failure;
-      break;
-    }
-
     case CTO_Contraction:
     case CTO_Literal: {
-      ByteString find;
+      CharacterString find;
       if (getFindText(data, &find, &token, &length))
         if (!addRule(data, opcode, &find, NULL, after, before))
           goto failure;
@@ -654,60 +758,72 @@ doOpcode:
     case CTO_CapitalSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "capital sign"))
-        if (parseDots(data, &cells, token, length))
-          if (!saveByteString(data, &tableHeader->capitalSign, &cells))
-            goto failure;
+        if (parseDots(data, &cells, token, length)) {
+          ContractionTableOffset offset;
+          if (!saveSequence(data, &offset, &cells)) goto failure;
+          tableHeader->capitalSign = offset;
+        }
       break;
     }
 
     case CTO_BeginCapitalSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "begin capital sign"))
-        if (parseDots(data, &cells, token, length))
-          if (!saveByteString(data, &tableHeader->beginCapitalSign, &cells))
-            goto failure;
+        if (parseDots(data, &cells, token, length)) {
+          ContractionTableOffset offset;
+          if (!saveSequence(data, &offset, &cells)) goto failure;
+          tableHeader->beginCapitalSign = offset;
+        }
       break;
     }
 
     case CTO_EndCapitalSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "end capital sign"))
-        if (parseDots(data, &cells, token, length))
-          if (!saveByteString(data, &tableHeader->endCapitalSign, &cells))
-            goto failure;
+        if (parseDots(data, &cells, token, length)) {
+          ContractionTableOffset offset;
+          if (!saveSequence(data, &offset, &cells)) goto failure;
+          tableHeader->endCapitalSign = offset;
+        }
       break;
     }
 
     case CTO_EnglishLetterSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "letter sign"))
-        if (parseDots(data, &cells, token, length))
-          if (!saveByteString(data, &tableHeader->englishLetterSign, &cells))
-            goto failure;
+        if (parseDots(data, &cells, token, length)) {
+          ContractionTableOffset offset;
+          if (!saveSequence(data, &offset, &cells)) goto failure;
+          tableHeader->englishLetterSign = offset;
+        }
       break;
     }
 
     case CTO_NumberSign: {
       ByteString cells;
       if (getToken(data, &token, &length, "number sign"))
-        if (parseDots(data, &cells, token, length))
-          if (!saveByteString(data, &tableHeader->numberSign, &cells))
-            goto failure;
+        if (parseDots(data, &cells, token, length)) {
+          ContractionTableOffset offset;
+          if (!saveSequence(data, &offset, &cells)) goto failure;
+          tableHeader->numberSign = offset;
+        }
       break;
     }
 
     case CTO_Class: {
       const struct CharacterClass *class;
-      ByteString characters;
+      CharacterString characters;
       if (getToken(data, &token, &length, "character class name")) {
         if ((class = findCharacterClass(token, length))) {
-          compileError(data, "character class already defined: %.*s", length, token);
+          compileError(data, "character class already defined: %.*" PRIws, length, token);
         } else if ((class = addCharacterClass(data, token, length))) {
           if (getCharacters(data, &characters, &token, &length)) {
             int index;
             for (index=0; index<characters.length; ++index) {
-              ContractionTableCharacter *character = &tableHeader->characters[characters.bytes[index]];
-              character->attributes |= class->attribute;
+              wchar_t character = characters.characters[index];
+              ContractionTableCharacter *entry = getCharacterEntry(character);
+              if (!entry) goto failure;
+              entry->attributes |= class->attribute;
             }
           }
         }
@@ -734,14 +850,40 @@ doOpcode:
     }
 
     default:
-      compileError(data, "unimplemented opcode: %.*s", length, token);
+      compileError(data, "unimplemented opcode: %.*" PRIws, length, token);
       break;
   }				/*end of loop for processing tableStream */
   ok = 1;
 
 failure:
   return ok;
-}				/*compilation completed */
+}
+
+static int
+processUtf8Line (char *line, void *dataAddress) {
+  FileData *data = dataAddress;
+  size_t length = strlen(line);
+  const char *byte = line;
+  wchar_t characters[length+1];
+  wchar_t *character = characters;
+
+  data->lineNumber++;
+
+  while (length) {
+    const char *start = byte;
+    wint_t wc = convertUtf8ToWchar(&byte, &length);
+
+    if (wc == WEOF) {
+      compileError(data, "illegal UTF-8 character at offset %d", start-line);
+      return 1;
+    }
+
+    *character++ = wc;
+  }
+
+  *character = 0;
+  return processWcharLine(data, characters);
+}
 
 static int
 processFile (const char *fileName) {
@@ -754,52 +896,7 @@ processFile (const char *fileName) {
   LogPrint(LOG_DEBUG, "Including contraction table: %s", fileName);
 
   if ((stream = openDataFile(data.fileName, "r"))) {
-    int size = 0X80;
-    char *buffer = malloc(size);
-    if (buffer) {
-      while (1) { /*Process lines in table */
-        char *line = fgets(buffer, size, stream);
-        int length;
-        if (!line) {
-          if (ferror(stream))
-            compileError(&data, "read error: %s", strerror(errno));
-          break;
-        }
-        data.lineNumber++;
-
-        while (buffer[(length = strlen(line)) - 1] != '\n') {
-          if (length + 1 == size) {
-            char *newBuffer = realloc(buffer, size<<=1);
-            if (!newBuffer) {
-              compileError(&data, "cannot extend input buffer.");
-              ok = 0;
-              goto done;
-            }
-            buffer = newBuffer;
-          }
-          if (!(line = fgets(buffer+length, size-length, stream))) {
-            if (ferror(stream))
-              compileError(&data, "read error: %s", strerror(errno));
-            else
-              compileError(&data, "unterminated final line.");
-            goto done;
-          }
-          length += strlen(line);
-          line = buffer;
-        }
-        buffer[length-1] = 0;
-
-        if (!processLine(&data, line)) {
-          ok = 0;
-          break;
-        }
-      }				/*end of loop for processing tableStream */
-    done:
-      free(buffer);
-    } else {
-      compileError(&data, "cannot allocate input buffer.");
-      ok = 0;
-    }
+    if (!processLines(stream, processUtf8Line, &data)) return 0;
     fclose(stream);
   } else {
     LogPrint(LOG_ERR,
@@ -809,81 +906,8 @@ processFile (const char *fileName) {
   return ok;
 }				/*compilation completed */
 
-static void
-cacheCharacterAttributes (void) {
-  int character;
-  for (character = 0; character < 0X100; character++) {
-    ContractionTableCharacter *c = &tableHeader->characters[character];
-    c->uppercase = c->lowercase = character;
-    if (isspace(character))
-      c->attributes |= CTC_Space;
-    else if (isdigit(character))
-      c->attributes |= CTC_Digit;
-    else if (ispunct(character))
-      c->attributes |= CTC_Punctuation;
-    else if (isalpha(character)) {
-      c->attributes |= CTC_Letter;
-      if (isupper(character)) {
-        c->attributes |= CTC_UpperCase;
-        c->lowercase = tolower(character);
-      } else if (islower(character)) {
-        c->attributes |= CTC_LowerCase;
-        c->uppercase = toupper(character);
-      }
-    }
-  }
-}
-
-static int
-auditCharacters (BYTE *encountered, const BYTE *start, BYTE length) {
-   const BYTE *address = start;
-   BYTE count = length;
-  while (count--) {
-    BYTE character = *address++;
-    if (!tableHeader->characters[character].always && !encountered[character]) {
-      encountered[character] = 1;
-      compileError(NULL, "character representation not defined: %c (%.*s)",
-                   character, length, start);
-      return 0;
-    }
-  }
-  return 1;
-}
-
-static int
-auditTable (void) {
-  int ok = 1;
-  BYTE characterEncountered[0X100];
-  int hashIndex;
-  memset(characterEncountered, 0, sizeof(characterEncountered));
-  for (hashIndex = 0; hashIndex < HASHNUM; hashIndex++) {
-    ContractionTableOffset bucket = tableHeader->rules[hashIndex];
-    while (bucket) {
-      const ContractionTableRule *rule = CTR(tableHeader, bucket);
-      switch (rule->opcode) {
-        default:
-          if (!rule->replen) {
-            if (!auditCharacters(characterEncountered, &rule->findrep[0], rule->findlen)) {
-              ok = 0;
-            }
-          }
-          break;
-        case CTO_Replace:
-          if (!auditCharacters(characterEncountered, &rule->findrep[rule->findlen], rule->replen)) {
-            ok = 0;
-          }
-          break;
-        case CTO_Literal:
-          break;
-      }
-      bucket = rule->next;
-    }
-  }
-  return ok;
-}
-
-void *
-compileContractionTable (const char *fileName) { /*compile source table into a table in memory */
+ContractionTable *
+compileContractionTable (const char *fileName) {
   int ok = 0;
   ContractionTableOffset headerOffset;
   errorCount = 0;
@@ -892,34 +916,25 @@ compileContractionTable (const char *fileName) { /*compile source table into a t
   tableSize = 0;
   tableUsed = 0;
 
+  characterTable = NULL;
+  characterTableSize = 0;
+  characterEntryCount = 0;
+
   if (!opcodeLengths[0]) {
     ContractionTableOpcode opcode;
     for (opcode=0; opcode<CTO_None; ++opcode)
-      opcodeLengths[opcode] = strlen(opcodeNames[opcode]);
+      opcodeLengths[opcode] = wcslen(opcodeNames[opcode]);
   }
 
-  {
-    const char *locale = getLocale();
-    originalLocale = locale? strdup(locale): NULL;
-  }
-  setLocale("C");
-  noLocale = 0;
-
   if (allocateBytes(NULL, &headerOffset, sizeof(*tableHeader), __alignof__(*tableHeader))) {
     if (headerOffset == 0) {
       if (allocateCharacterClasses()) {
         if (processFile(fileName)) {
-          if (auditTable()) {
+          if (saveCharacterTable()) {
             ok = 1;
-            if (!noLocale) {
-              if (saveString(NULL, &tableHeader->locale, getLocale())) {
-                cacheCharacterAttributes();
-              } else {
-                ok = 0;
-              }
-            }
           }
         }
+
         deallocateCharacterClasses();
       }
     } else {
@@ -927,27 +942,34 @@ compileContractionTable (const char *fileName) { /*compile source table into a t
     }
   }
 
-  if (originalLocale) {
-    setLocale(originalLocale);
-    free(originalLocale);
-    originalLocale = NULL;
-  }
+  if (characterTable) free(characterTable);
 
-  if (!ok) {
-    free(tableHeader);
-    tableHeader = NULL;
-  }
-
-  if (errorCount)
+  if (errorCount) {
     LogPrint(LOG_WARNING, "%d %s in contraction table '%s'.",
              errorCount, ((errorCount == 1)? "error": "errors"), fileName);
-  return (void *)tableHeader;
+  }
+
+  if (ok) {
+    ContractionTable *table;
+
+    if ((table = malloc(sizeof(*table)))) {
+      table->header = tableHeader;
+      table->characters = NULL;
+      table->charactersSize = 0;
+      table->characterCount = 0;
+      return table;
+    }
+  }
+
+  if (tableHeader) free(tableHeader);
+  return NULL;
 }
 
 int
-destroyContractionTable (void *contractionTable) {
-  ContractionTableHeader *table = (ContractionTableHeader *) contractionTable;
-  free(table);
+destroyContractionTable (ContractionTable *contractionTable) {
+  if (contractionTable->characters) free(contractionTable->characters);
+  if (contractionTable->header) free(contractionTable->header);
+  free(contractionTable);
   return 1;
 }
 
