@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -29,20 +29,19 @@ extern "C" {
 #include <sys/time.h>
 
 /* These macros are meant for internal use only. */
-#define BITMASK_ELEMENT_TYPE unsigned char
-#define BITMASK_ELEMENT_SIZE (sizeof(BITMASK_ELEMENT_TYPE) * 8)
-#define BITMASK_INDEX(bit) ((bit) / BITMASK_ELEMENT_SIZE)
-#define BITMASK_SHIFT(bit) ((bit) % BITMASK_ELEMENT_SIZE)
-#define BITMASK_ELEMENT_COUNT(bits) BITMASK_INDEX((bits) + 1)
-#define BITMASK_ELEMENT(mask,bit) ((mask)[BITMASK_INDEX((bit))])
-#define BITMASK_BIT(bit) (1 << BITMASK_SHIFT((bit)))
+#define BITMASK_ELEMENT_SIZE(element) (sizeof(element) * 8)
+#define BITMASK_INDEX(bit,size) ((bit) / (size))
+#define BITMASK_SHIFT(bit,size) ((bit) % (size))
+#define BITMASK_ELEMENT_COUNT(bits,size) (BITMASK_INDEX((bits)-1, (size)) + 1)
+#define BITMASK_ELEMENT(name,bit) ((name)[BITMASK_INDEX((bit), BITMASK_ELEMENT_SIZE((name)[0]))])
+#define BITMASK_BIT(name,bit) (1 << BITMASK_SHIFT((bit), BITMASK_ELEMENT_SIZE((name)[0])))
 
 /* These macros are for public use. */
-#define BITMASK(name,bits) BITMASK_ELEMENT_TYPE name[BITMASK_ELEMENT_COUNT((bits))]
-#define BITMASK_SIZE(mask) (sizeof((mask)) * 8)
-#define BITMASK_SET(mask,bit) (BITMASK_ELEMENT((mask), (bit)) |= BITMASK_BIT((bit)))
-#define BITMASK_CLEAR(mask,bit) (BITMASK_ELEMENT((mask), (bit)) &= ~BITMASK_BIT((bit)))
-#define BITMASK_TEST(mask,bit) (BITMASK_ELEMENT((mask), (bit)) & BITMASK_BIT((bit)))
+#define BITMASK(name,bits,type) unsigned type name[BITMASK_ELEMENT_COUNT((bits), BITMASK_ELEMENT_SIZE(type))]
+#define BITMASK_SIZE(name) BITMASK_ELEMENT_SIZE((name))
+#define BITMASK_SET(name,bit) (BITMASK_ELEMENT((name), (bit)) |= BITMASK_BIT((name), (bit)))
+#define BITMASK_CLEAR(name,bit) (BITMASK_ELEMENT((name), (bit)) &= ~BITMASK_BIT((name), (bit)))
+#define BITMASK_TEST(name,bit) (BITMASK_ELEMENT((name), (bit)) & BITMASK_BIT((name), (bit)))
 
 #ifndef MIN
 #define MIN(a, b)  (((a) < (b))? (a): (b)) 
@@ -218,6 +217,9 @@ extern int validateFlag (unsigned int *value, const char *string, const char *on
 extern int validateOnOff (unsigned int *value, const char *string);
 extern int validateYesNo (unsigned int *value, const char *string);
 
+extern char **getParameters (const char *const *names, const char *qualifier, const char *parameters);
+extern void logParameters (const char *const *names, char **values, char *description);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
