@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,11 +24,19 @@ extern "C" {
 
 #include "driver.h"
 
-/* mode argument for readScreen() */
-typedef enum {
-  SCR_TEXT,		/* get screen text */
-  SCR_ATTRIB		/* get screen attributes */
-} ScreenCharacterProperty;
+#define SCR_ATTR_FG_BLUE   0X01
+#define SCR_ATTR_FG_GREEN  0X02
+#define SCR_ATTR_FG_RED    0X04
+#define SCR_ATTR_FG_BRIGHT 0X08
+#define SCR_ATTR_BG_BLUE   0X10
+#define SCR_ATTR_BG_GREEN  0X20
+#define SCR_ATTR_BG_RED    0X40
+#define SCR_ATTR_BLINK     0X80
+
+typedef struct {
+  wchar_t text;
+  unsigned char attributes;
+} ScreenCharacter;
 
 typedef struct {
   short rows, cols;	/* screen dimensions */
@@ -44,11 +52,24 @@ typedef struct {
 } ScreenBox;
 
 extern int validateScreenBox (const ScreenBox *box, int columns, int rows);
-extern void setScreenMessage (const ScreenBox *box, unsigned char *buffer, ScreenCharacterProperty property, const char *message);
+extern void setScreenMessage (const ScreenBox *box, ScreenCharacter *buffer, const char *message);
+
+extern void clearScreenCharacters (ScreenCharacter *characters, size_t count);
+extern void setScreenCharacterText (ScreenCharacter *characters, char text, size_t count);
+extern void copyScreenCharacterText (ScreenCharacter *characters, const char *text, size_t count);
+extern void setScreenCharacterAttributes (ScreenCharacter *characters, unsigned char attributes, size_t count);
+
+#define SCR_KEY_SHIFT     0X40000000
+#define SCR_KEY_UPPER     0X20000000
+#define SCR_KEY_CONTROL   0X10000000
+#define SCR_KEY_ALT_LEFT  0X08000000
+#define SCR_KEY_ALT_RIGHT 0X04000000
+#define SCR_KEY_CHAR_MASK 0X00FFFFFF
+
+#define SCR_KEY_UNICODE_ROW 0XF800
 
-#define SCR_KEY_MOD_META 0X100
 typedef enum {
-  SCR_KEY_ENTER = 0X200,
+  SCR_KEY_ENTER = SCR_KEY_UNICODE_ROW,
   SCR_KEY_TAB,
   SCR_KEY_BACKSPACE,
   SCR_KEY_ESCAPE,
@@ -83,10 +104,9 @@ extern void deactivateHelpScreen (void);
 
 /* Routines which apply to the current screen. */
 extern void describeScreen (ScreenDescription *);		/* get screen status */
-extern int readScreen (short left, short top, short width, short height, unsigned char *buffer, ScreenCharacterProperty);
-extern int insertKey (ScreenKey key);
-extern int insertCharacters (const char *, int);
-extern int insertString (const char *);
+extern int readScreen (short left, short top, short width, short height, ScreenCharacter *buffer);
+extern int readScreenText (short left, short top, short width, short height, wchar_t *buffer);
+extern int insertScreenKey (ScreenKey key);
 extern int routeCursor (int column, int row, int screen);
 extern int highlightScreenRegion (int left, int right, int top, int bottom);
 extern int unhighlightScreenRegion (void);
