@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -24,6 +24,64 @@
 #include "scr.h"
 #include "scr_base.h"
 
+int
+isSpecialKey (ScreenKey key) {
+  return (key & (SCR_KEY_CHAR_MASK & ~0XFF)) == SCR_KEY_UNICODE_ROW;
+}
+
+void
+setKeyModifiers (ScreenKey *key, ScreenKey which) {
+  if (!isSpecialKey(*key)) {
+    wchar_t character = *key & SCR_KEY_CHAR_MASK;
+    ScreenKey modifiers = *key & ~SCR_KEY_CHAR_MASK;
+
+    if (which & (SCR_KEY_UPPER | SCR_KEY_SHIFT)) {
+      if (!(modifiers & (SCR_KEY_UPPER | SCR_KEY_SHIFT))) {
+        if (iswupper(character)) {
+          character = towlower(character);
+
+          if (which & SCR_KEY_UPPER) {
+            modifiers |= SCR_KEY_UPPER;
+          } else {
+            modifiers |= SCR_KEY_SHIFT;
+          }
+        }
+      }
+    } else {
+      if (modifiers & (SCR_KEY_UPPER | SCR_KEY_SHIFT)) {
+        if (iswalpha(character)) {
+          character = towupper(character);
+          modifiers &= ~SCR_KEY_SHIFT;
+        }
+
+        modifiers &= ~SCR_KEY_UPPER;
+      }
+    }
+
+    if (which & SCR_KEY_CONTROL) {
+      if (!(modifiers & SCR_KEY_CONTROL)) {
+        if (character < 0X20) {
+          character |= 0X60;
+          modifiers |= SCR_KEY_CONTROL;
+        }
+      }
+    } else {
+      if (modifiers & SCR_KEY_CONTROL) {
+        if (iswLatin1(character)) {
+          if ((character & 0X6F) == 0X2F) {
+            character |= 0X50;
+          } else {
+            character &= 0X9F;
+          }
+        }
+        modifiers &= ~SCR_KEY_CONTROL;
+      }
+    }
+
+    *key = character | modifiers;
+  }
+}
+
 static const char *text_BaseScreen = "no screen";
 
 static int
@@ -51,15 +109,11 @@ describe_BaseScreen (ScreenDescription *description) {
 }
 
 static int
-read_BaseScreen (ScreenBox box, unsigned char *buffer, ScreenCharacterProperty property) {
+readCharacters_BaseScreen (const ScreenBox *box, ScreenCharacter *buffer) {
   ScreenDescription description;
   describe_BaseScreen(&description);
-  if (!validateScreenBox(&box, description.cols, description.rows)) return 0;
-  if (property == SCR_TEXT) {
-    memcpy(buffer, text_BaseScreen+box.left, box.width);
-  } else {
-    memset(buffer, 0X07, box.width);
-  }
+  if (!validateScreenBox(box, description.cols, description.rows)) return 0;
+  setScreenMessage(box, buffer, text_BaseScreen);
   return 1;
 }
 
@@ -99,7 +153,7 @@ initializeBaseScreen (BaseScreen *base) {
   base->switchVirtualTerminal = switchVirtualTerminal_BaseScreen;
   base->currentVirtualTerminal = currentVirtualTerminal_BaseScreen;
   base->describe = describe_BaseScreen;
-  base->read = read_BaseScreen;
+  base->readCharacters = readCharacters_BaseScreen;
   base->insertKey = insertKey_BaseScreen;
   base->routeCursor = routeCursor_BaseScreen;
   base->highlightRegion = highlightRegion_BaseScreen;
