@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -26,8 +26,8 @@ extern "C" {
 
 typedef struct {
   BaseScreen base;
-  int (*open) (BaseScreen *);		/* called every time the screen is frozen */
-  void (*close) (void);		/* called to discard frozen screen image */
+  int (*construct) (BaseScreen *);		/* called every time the screen is frozen */
+  void (*destruct) (void);		/* called to discard frozen screen image */
 } FrozenScreen;
 
 extern void initializeFrozenScreen (FrozenScreen *);
