@@ -1,8 +1,8 @@
 /*
- * BRLTTY - A background process providing access to the Linux console (when in
+ * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2004 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -15,12 +15,8 @@
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif /* HAVE_CONFIG_H */
+#include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -31,10 +27,18 @@
 #include "misc.h"
 #include "system.h"
 
+#include "sys_prog_none.h"
+
 #include "sys_boot_none.h"
 
+#include "sys_exec_unix.h"
+
+#include "sys_mount_none.h"
+
+#ifdef ENABLE_SHARED_OBJECTS
 #define SHARED_OBJECT_LOAD_FLAGS (DL_LAZY)
 #include "sys_shlib_dlfcn.h"
+#endif /* ENABLE_SHARED_OBJECTS */
 
 #include "sys_beep_wskbd.h"
 
@@ -48,3 +52,5 @@
 #endif /* ENABLE_MIDI_SUPPORT */
 
 #include "sys_ports_none.h"
+
+#include "sys_kbd_none.h"
