@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -17,20 +17,15 @@
 
 #include "prologue.h"
 
-#include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
 #include <dirent.h>
 #include <fcntl.h>
-#include <mntent.h>
-#include <sys/mount.h>
 #include <sys/stat.h>
 #include <sys/vfs.h>
 #include <sys/ioctl.h>
-#include <linux/compiler.h>
 #include <linux/usbdevice_fs.h>
 
 #ifndef USBDEVFS_DISCONNECT
@@ -42,48 +37,77 @@
 #endif /* USBDEVFS_CONNECT */
 
 #include "misc.h"
+#include "mount.h"
 #include "io_usb.h"
 #include "usb_internal.h"
 
-typedef struct {
-  char *path;
-  int file;
-} UsbDeviceExtension;
+struct UsbDeviceExtensionStruct {
+  char *usbfsPath;
+  char *sysfsPath;
+  int usbfsFile;
+};
 
-typedef struct {
+struct UsbEndpointExtensionStruct {
   Queue *completedRequests;
-} UsbEndpointExtension;
+};
 
 int
 usbResetDevice (UsbDevice *device) {
   UsbDeviceExtension *devx = device->extension;
-  if (ioctl(devx->file, USBDEVFS_RESET, NULL) != -1) return 1;
+  if (ioctl(devx->usbfsFile, USBDEVFS_RESET, NULL) != -1) return 1;
   LogError("USB device reset");
   return 0;
 }
 
 int
-usbSetConfiguration (
-  UsbDevice *device,
-  unsigned char configuration
-) {
+usbDisableAutosuspend (UsbDevice *device) {
   UsbDeviceExtension *devx = device->extension;
-  unsigned int arg = configuration;
-  if (ioctl(devx->file, USBDEVFS_SETCONFIGURATION, &arg) != -1) return 1;
-  LogError("USB configuration set");
-  return 0;
+  int ok = 0;
+  char *path = makePath(devx->sysfsPath, "device/power/autosuspend");
+
+  if (path) {
+    int file = open(path, O_WRONLY);
+
+    if (file != -1) {
+      static const char *const values[] = {"-1", "0", NULL};
+      const char *const *value = values;
+
+      while (*value) {
+        size_t length = strlen(*value);
+        ssize_t result = write(file, *value, length);
+
+        if (result != -1) {
+          ok = 1;
+          break;
+        }
+
+        if (errno != EINVAL) {
+          LogPrint(LOG_ERR, "write error: %s: %s", path, strerror(errno));
+          break;
+        }
+
+        ++value;
+      }
+
+      close(file);
+    } else {
+      LogPrint((errno == ENOENT)? LOG_DEBUG: LOG_ERR,
+               "open error: %s: %s", path, strerror(errno));
+    }
+
+    free(path);
+  }
+
+  return ok;
 }
 
 static char *
-usbGetDriver (
-  UsbDevice *device,
-  unsigned char interface
-) {
+usbGetDriver (UsbDevice *device, unsigned char interface) {
   UsbDeviceExtension *devx = device->extension;
   struct usbdevfs_getdriver arg;
   memset(&arg, 0, sizeof(arg));
   arg.interface = interface;
-  if (ioctl(devx->file, USBDEVFS_GETDRIVER, &arg) == -1) return NULL;
+  if (ioctl(devx->usbfsFile, USBDEVFS_GETDRIVER, &arg) == -1) return NULL;
   return strdup(arg.driver);
 }
 
@@ -100,16 +124,13 @@ usbControlDriver (
   arg.ifno = interface;
   arg.ioctl_code = code;
   arg.data = data;
-  if (ioctl(devx->file, USBDEVFS_IOCTL, &arg) != -1) return 1;
+  if (ioctl(devx->usbfsFile, USBDEVFS_IOCTL, &arg) != -1) return 1;
   LogError("USB driver control");
   return 0;
 }
 
 static int
-usbDisconnectDriver (
-  UsbDevice *device,
-  unsigned char interface
-) {
+usbDisconnectDriver (UsbDevice *device, unsigned char interface) {
 #ifdef USBDEVFS_DISCONNECT
   if (usbControlDriver(device, interface, USBDEVFS_DISCONNECT, NULL)) return 1;
 #else /* USBDEVFS_DISCONNECT */
@@ -118,29 +139,80 @@ usbDisconnectDriver (
   return 0;
 }
 
+static int
+usbDisconnectInterface (UsbDevice *device, unsigned char interface) {
+  char *driver = usbGetDriver(device, interface);
+
+  if (driver) {
+    LogPrint(LOG_WARNING, "USB interface in use: %s", driver);
+    free(driver);
+
+    if (usbDisconnectDriver(device, interface)) return 1;
+  }
+
+  return 0;
+}
+
+static int
+usbDisconnectInterfaces (UsbDevice *device) {
+  const UsbDescriptor *descriptor = NULL;
+  int disconnected = 0;
+
+  while (usbNextDescriptor(device, &descriptor)) {
+    if (descriptor->interface.bDescriptorType == UsbDescriptorType_Interface) {
+      if (usbDisconnectInterface(device, descriptor->interface.bInterfaceNumber)) disconnected = 1;
+    }
+  }
+
+  return disconnected;
+}
+
+int
+usbSetConfiguration (
+  UsbDevice *device,
+  unsigned char configuration
+) {
+  UsbDeviceExtension *devx = device->extension;
+  int disconnected = 0;
+
+  while (1) {
+    unsigned int arg = configuration;
+
+    if (ioctl(devx->usbfsFile, USBDEVFS_SETCONFIGURATION, &arg) != -1) return 1;
+    if (errno != EBUSY) break;
+    if (disconnected) break;
+
+    if (!usbDisconnectInterfaces(device)) {
+      errno = EBUSY;
+      break;
+    }
+    disconnected = 1;
+  }
+
+  LogError("USB configuration set");
+  return 0;
+}
+
 int
 usbClaimInterface (
   UsbDevice *device,
   unsigned char interface
 ) {
   UsbDeviceExtension *devx = device->extension;
-  unsigned int arg = interface;
   int disconnected = 0;
-claim:
-  if (ioctl(devx->file, USBDEVFS_CLAIMINTERFACE, &arg) != -1) return 1;
 
-  if (errno == EBUSY) {
-    char *driver = usbGetDriver(device, interface);
-    if (driver) {
-      LogPrint(LOG_WARNING, "USB interface in use: %s", driver);
-      free(driver);
+  while (1) {
+    unsigned int arg = interface;
 
-      if (!disconnected) {
-        disconnected = 1;
-        if (usbDisconnectDriver(device, interface)) goto claim;
-      }
+    if (ioctl(devx->usbfsFile, USBDEVFS_CLAIMINTERFACE, &arg) != -1) return 1;
+    if (errno != EBUSY) break;
+    if (disconnected) break;
+
+    if (!usbDisconnectInterface(device, interface)) {
+      errno = EBUSY;
+      break;
     }
-    errno = EBUSY;
+    disconnected = 1;
   }
 
   LogError("USB interface claim");
@@ -154,7 +226,8 @@ usbReleaseInterface (
 ) {
   UsbDeviceExtension *devx = device->extension;
   unsigned int arg = interface;
-  if (ioctl(devx->file, USBDEVFS_RELEASEINTERFACE, &arg) != -1) return 1;
+  if (ioctl(devx->usbfsFile, USBDEVFS_RELEASEINTERFACE, &arg) != -1) return 1;
+  if (errno == ENODEV) return 1;
   LogError("USB interface release");
   return 0;
 }
@@ -170,7 +243,7 @@ usbSetAlternative (
   memset(&arg, 0, sizeof(arg));
   arg.interface = interface;
   arg.altsetting = alternative;
-  if (ioctl(devx->file, USBDEVFS_SETINTERFACE, &arg) != -1) return 1;
+  if (ioctl(devx->usbfsFile, USBDEVFS_SETINTERFACE, &arg) != -1) return 1;
   LogError("USB alternative set");
   return 0;
 }
@@ -182,7 +255,7 @@ usbResetEndpoint (
 ) {
   UsbDeviceExtension *devx = device->extension;
   unsigned int arg = endpointAddress;
-  if (ioctl(devx->file, USBDEVFS_RESETEP, &arg) != -1) return 1;
+  if (ioctl(devx->usbfsFile, USBDEVFS_RESETEP, &arg) != -1) return 1;
   LogError("USB endpoint reset");
   return 0;
 }
@@ -194,7 +267,7 @@ usbClearEndpoint (
 ) {
   UsbDeviceExtension *devx = device->extension;
   unsigned int arg = endpointAddress;
-  if (ioctl(devx->file, USBDEVFS_CLEAR_HALT, &arg) != -1) return 1;
+  if (ioctl(devx->usbfsFile, USBDEVFS_CLEAR_HALT, &arg) != -1) return 1;
   LogError("USB endpoint clear");
   return 0;
 }
@@ -227,7 +300,7 @@ usbControlTransfer (
   arg.transfer.timeout = timeout;
 
   {
-    int count = ioctl(devx->file, USBDEVFS_CONTROL, &arg);
+    int count = ioctl(devx->usbfsFile, USBDEVFS_CONTROL, &arg);
     if (count == -1) LogError("USB control transfer");
     return count;
   }
@@ -241,7 +314,7 @@ usbReapUrb (
   UsbDeviceExtension *devx = device->extension;
   struct usbdevfs_urb *urb;
 
-  if (ioctl(devx->file,
+  if (ioctl(devx->usbfsFile,
             wait? USBDEVFS_REAPURB: USBDEVFS_REAPURBNDELAY,
             &urb) != -1) {
     if (urb) {
@@ -305,7 +378,7 @@ usbSubmitRequest (
                urb->buffer, urb->buffer_length, urb->usercontext);
     */
     submit:
-      if (ioctl(devx->file, USBDEVFS_SUBMITURB, urb) != -1) return urb;
+      if (ioctl(devx->usbfsFile, USBDEVFS_SUBMITURB, urb) != -1) return urb;
       if ((errno == EINVAL) &&
           (USB_ENDPOINT_TRANSFER(endpoint->descriptor) == UsbEndpointTransfer_Interrupt) &&
           (urb->type == USBDEVFS_URB_TYPE_BULK)) {
@@ -330,10 +403,15 @@ usbCancelRequest (
   void *request
 ) {
   UsbDeviceExtension *devx = device->extension;
+  int reap = 1;
 
-  if (ioctl(devx->file, USBDEVFS_DISCARDURB, request) == -1)
-    if (errno != EINVAL)
+  if (ioctl(devx->usbfsFile, USBDEVFS_DISCARDURB, request) == -1) {
+    if (errno == ENODEV)  {
+      reap = 0;
+    } else if (errno != EINVAL) {
       LogError("USB URB discard");
+    }
+  }
   
   {
     struct usbdevfs_urb *urb = request;
@@ -344,6 +422,7 @@ usbCancelRequest (
       int found = 1;
 
       while (!deleteItem(eptx->completedRequests, request)) {
+        if (!reap) break;
         if (!usbReapUrb(device, 0)) {
           found = 0;
           break;
@@ -424,7 +503,7 @@ usbBulkTransfer (
   arg.timeout = timeout;
 
   {
-    int count = ioctl(devx->file, USBDEVFS_BULK, &arg);
+    int count = ioctl(devx->usbfsFile, USBDEVFS_BULK, &arg);
     if (count != -1) return count;
     LogError("USB bulk transfer");
   }
@@ -554,7 +633,7 @@ usbWriteEndpoint (
 int
 usbReadDeviceDescriptor (UsbDevice *device) {
   UsbDeviceExtension *devx = device->extension;
-  int count = read(devx->file, &device->descriptor, UsbDescriptorSize_Device);
+  int count = read(devx->usbfsFile, &device->descriptor, UsbDescriptorSize_Device);
   if (count == -1) {
     LogError("USB device descriptor read");
   } else if (count != UsbDescriptorSize_Device) {
@@ -584,9 +663,7 @@ usbAllocateEndpointExtension (UsbEndpoint *endpoint) {
 }
 
 void
-usbDeallocateEndpointExtension (UsbEndpoint *endpoint) {
-  UsbEndpointExtension *eptx = endpoint->extension;
-
+usbDeallocateEndpointExtension (UsbEndpointExtension *eptx) {
   if (eptx->completedRequests) {
     deallocateQueue(eptx->completedRequests);
     eptx->completedRequests = NULL;
@@ -596,62 +673,119 @@ usbDeallocateEndpointExtension (UsbEndpoint *endpoint) {
 }
 
 void
-usbDeallocateDeviceExtension (UsbDevice *device) {
-  UsbDeviceExtension *devx = device->extension;
-
-  if (devx->file != -1) {
-    close(devx->file);
-    devx->file = -1;
+usbDeallocateDeviceExtension (UsbDeviceExtension *devx) {
+  if (devx->usbfsFile != -1) {
+    close(devx->usbfsFile);
+    devx->usbfsFile = -1;
   }
 
-  if (devx->path) {
-    free(devx->path);
-    devx->path = NULL;
+  if (devx->usbfsPath) {
+    free(devx->usbfsPath);
+    devx->usbfsPath = NULL;
+  }
+
+  if (devx->sysfsPath) {
+    free(devx->sysfsPath);
+    devx->sysfsPath = NULL;
   }
 
   free(devx);
 }
 
+static char *
+usbMakeSysfsPath (const char *usbfsPath) {
+  const char *tail = usbfsPath + strlen(usbfsPath);
+
+  {
+    int count = 0;
+    while (1) {
+      if (tail == usbfsPath) return NULL;
+      if (!isPathDelimiter(*--tail)) continue;
+      if (++count == 2) break;
+    }
+  }
+
+  {
+    unsigned int bus;
+    unsigned int device;
+    char extra;
+    int count = sscanf(tail, "/%u/%u%c", &bus, &device, &extra);
+
+    if (count == 2) {
+      const char *format = "/sys/class/usb_device/usbdev%u.%u";
+      char buffer[strlen(format) + (2 * 0X10) + 1];
+      snprintf(buffer, sizeof(buffer), format, bus, device);
+
+      {
+        char *sysfsPath = strdup(buffer);
+        if (sysfsPath) return sysfsPath;
+      }
+    }
+  }
+
+  return NULL;
+}
+
 static UsbDevice *
 usbSearchDevice (const char *root, UsbDeviceChooser chooser, void *data) {
   size_t rootLength = strlen(root);
   UsbDevice *device = NULL;
   DIR *directory;
+
   if ((directory = opendir(root))) {
     struct dirent *entry;
+
     while ((entry = readdir(directory))) {
       size_t nameLength = strlen(entry->d_name);
       struct stat status;
       char path[rootLength + 1 + nameLength + 1];
-      if (nameLength != 3) continue;
-      if (!isdigit(entry->d_name[0]) ||
-          !isdigit(entry->d_name[1]) ||
-          !isdigit(entry->d_name[2])) continue;
+
+      if (strspn(entry->d_name, "0123456789") != nameLength) continue;
       snprintf(path, sizeof(path), "%s/%s", root, entry->d_name);
       if (stat(path, &status) == -1) continue;
+
       if (S_ISDIR(status.st_mode)) {
         if ((device = usbSearchDevice(path, chooser, data))) break;
-      } else if (S_ISREG(status.st_mode)) {
+      } else if (S_ISREG(status.st_mode) || S_ISCHR(status.st_mode)) {
         UsbDeviceExtension *devx;
+
         if ((devx = malloc(sizeof(*devx)))) {
-          if ((devx->path = strdup(path))) {
-            if ((devx->file = open(devx->path, O_RDWR)) != -1) {
-              if ((device = usbTestDevice(devx, chooser, data))) break;
-              close(devx->file);
+          if ((devx->usbfsPath = strdup(path))) {
+            if ((devx->sysfsPath = usbMakeSysfsPath(devx->usbfsPath))) {
+              if ((devx->usbfsFile = open(devx->usbfsPath, O_RDWR)) != -1) {
+                if ((device = usbTestDevice(devx, chooser, data))) break;
+
+                close(devx->usbfsFile);
+                devx->usbfsFile = -1;
+              }
+
+              free(devx->sysfsPath);
+              devx->sysfsPath = NULL;
             }
-            free(devx->path);
+
+            free(devx->usbfsPath);
+            devx->usbfsPath = NULL;
           }
+
           free(devx);
         }
       }
     }
+
     closedir(directory);
   }
+
   return device;
 }
 
 static int
-usbTestPath (const char *path) {
+usbVerifyDirectory (const char *path) {
+  if (access(path, F_OK) != -1) return 1;
+  return 0;
+}
+
+static int
+usbVerifyUsbfs (const char *path) {
   struct statfs status;
   if (statfs(path, &status) != -1) {
     if (status.f_type == USBDEVICE_SUPER_MAGIC) return 1;
@@ -659,45 +793,56 @@ usbTestPath (const char *path) {
   return 0;
 }
 
+static int
+usbTestUsbfs (const char *path, const char *type) {
+  if ((strcmp(type, "usbdevfs") == 0) ||
+      (strcmp(type, "usbfs") == 0)) {
+    if (usbVerifyUsbfs(path)) {
+      return 1;
+    }
+  }
+  return 0;
+}
+
 static char *
 usbFindRoot (void) {
-  char *root = NULL;
-  const char *path = MOUNTED;
-  FILE *table;
-  if ((table = setmntent(path, "r"))) {
-    struct mntent *entry;
-    while ((entry = getmntent(table))) {
-      if ((strcmp(entry->mnt_type, "usbdevfs") == 0) ||
-          (strcmp(entry->mnt_type, "usbfs") == 0)) {
-        if (usbTestPath(entry->mnt_dir)) {
-          root = strdupWrapper(entry->mnt_dir);
-          break;
-        }
-      }
+  {
+    typedef struct {
+      const char *path;
+      int (*verify) (const char *path);
+    } RootEntry;
+
+    static const RootEntry roots[] = {
+      {.path="/dev/bus/usb", .verify=usbVerifyDirectory},
+      {.path="/proc/bus/usb", .verify=usbVerifyUsbfs},
+      {.path=NULL, .verify=NULL}
+    };
+    const RootEntry *root = roots;
+
+    while (root->path) {
+      LogPrint(LOG_DEBUG, "verifying USB device root: %s", root->path);
+      if (root->verify(root->path)) return strdupWrapper(root->path);
+      ++root;
     }
-    endmntent(table);
-  } else {
-    LogPrint((errno == ENOENT)? LOG_WARNING: LOG_ERR,
-             "Mounted file system table open erorr: %s: %s",
-             path, strerror(errno));
   }
-  return root;
+
+  return findMountPoint(usbTestUsbfs);
 }
 
 static char *
 usbMakeRoot (void) {
   const char *type = "usbfs";
-  const char *name = type;
-  char *directory = makePath(DATA_DIRECTORY, type);
+  char *directory = makeWritablePath(type);
+
   if (directory) {
     if (makeDirectory(directory)) {
-      if (usbTestPath(directory)) return directory;
-      LogPrint(LOG_NOTICE, "Mounting USBFS: %s", directory);
-      if (mount(name, directory, type, 0, NULL) != -1) return directory;
-      LogPrint(LOG_ERR, "USBFS mount error: %s: %s", directory, strerror(errno));
+      if (usbVerifyUsbfs(directory)) return directory;
+      if (makeMountPoint(directory, PACKAGE_NAME "-usbfs", type)) return directory;
     }
+
     free(directory);
   }
+
   return NULL;
 }
 
@@ -706,10 +851,11 @@ usbFindDevice (UsbDeviceChooser chooser, void *data) {
   UsbDevice *device = NULL;
   char *root;
   if ((root = usbFindRoot()) || (root = usbMakeRoot())) {
+    LogPrint(LOG_DEBUG, "USB Root: %s", root);
     device = usbSearchDevice(root, chooser, data);
     free(root);
   } else {
-    LogPrint(LOG_WARNING, "USBFS not mounted.");
+    LogPrint(LOG_DEBUG, "USBFS not mounted");
   }
   return device;
 }
