@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2005 by The BRLTTY Team. All rights reserved.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -22,9 +22,6 @@
 extern "C" {
 #endif /* __cplusplus */
 
-#define LINUX_SCREEN_DEVICES  "/dev/vcsa /dev/vcsa0 /dev/vcc/a"
-#define LINUX_CONSOLE_DEVICES "/dev/tty0 /dev/vc/0"
-
 typedef unsigned short int UnicodeNumber;
 typedef UnicodeNumber ApplicationCharacterMap[0X100];
 
