@@ -2,7 +2,7 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2007 by The BRLTTY Developers.
+ * Copyright (C) 1995-2008 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
@@ -21,59 +21,59 @@
 
 #include "misc.h"
 
-//#define SPK_HAVE_EXPRESS
 //#define SPK_HAVE_TRACK
 //#define SPK_HAVE_RATE
 //#define SPK_HAVE_VOLUME
+//#define SPK_HAVE_PUNCTUATION
 #include "spk_driver.h"
 
 static int
-spk_construct (char **parameters) {
+spk_construct (SpeechSynthesizer *spk, char **parameters) {
   return 0;
 }
 
 static void
-spk_destruct (void) {
+spk_destruct (SpeechSynthesizer *spk) {
 }
 
 static void
-spk_say (const unsigned char *buffer, int length) {
+spk_say (SpeechSynthesizer *spk, const unsigned char *buffer, int length, size_t count, const unsigned char *attributes) {
 }
 
 static void
-spk_mute (void) {
+spk_mute (SpeechSynthesizer *spk) {
 }
 
-#ifdef SPK_HAVE_EXPRESS
-static void
-spk_express (const unsigned char *buffer, int length) {
-}
-#endif /* SPK_HAVE_EXPRESS */
-
 #ifdef SPK_HAVE_TRACK
 static void
-spk_doTrack (void) {
+spk_doTrack (SpeechSynthesizer *spk) {
 }
 
 static int
-spk_getTrack (void) {
+spk_getTrack (SpeechSynthesizer *spk) {
   return 0;
 }
 
 static int
-spk_isSpeaking (void) {
+spk_isSpeaking (SpeechSynthesizer *spk) {
   return 0;
 }
 #endif /* SPK_HAVE_TRACK */
 
 #ifdef SPK_HAVE_RATE
 static void
-spk_rate (float setting) {
+spk_rate (SpeechSynthesizer *spk, unsigned char setting) {
 }
 #endif /* SPK_HAVE_RATE */
 
 #ifdef SPK_HAVE_VOLUME
 static void
-spk_volume (float setting) {
+spk_volume (SpeechSynthesizer *spk, unsigned char setting) {
 }
 #endif /* SPK_HAVE_VOLUME */
+
+#ifdef SPK_HAVE_PUNCTUATION
+static void
+spk_punctuation (SpeechSynthesizer *spk, SpeechPunctuation setting) {
+}
+#endif /* SPK_HAVE_PUNCTUATION */
